@@ -46,8 +46,6 @@ struct obj {
 	size_t buffer_size;
 	struct ring ring;
 	int (*header_func)(struct obj *self, GstBuffer *buf);
-	uint64_t next_timestamp;
-	int bps;
 };
 
 struct obj_class {
@@ -152,22 +150,12 @@ vorbis_header(struct obj *self, GstBuffer *buf)
 	return 1;
 }
 
-static int
-flac_header(struct obj *self, GstBuffer *buf)
-{
-	unsigned type = (buf->data[0] & 0x7F);
-	/* frame sync code */
-	if (type == 127)
-		return 0;
-	return 1;
-}
-
 static inline void
 calculate_timestamp(struct obj *self, GstBuffer *out_buf)
 {
 	uint32_t samples;
 
-	samples = out_buf->size / (self->av_ctx->channels * (self->bps >> 3));
+	samples = out_buf->size / (self->av_ctx->channels * sizeof(int16_t));
 
 	out_buf->timestamp = self->timestamp;
 	out_buf->duration = gst_util_uint64_scale_int(samples, GST_SECOND, self->av_ctx->sample_rate);
@@ -180,51 +168,34 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 {
 	struct obj *self;
 	GstFlowReturn ret = GST_FLOW_OK;
-	AVCodecContext *av_ctx;
 
 	self = (struct obj *)((GstObject *)pad)->parent;
-	av_ctx = self->av_ctx;
 
 	if (G_UNLIKELY(!self->got_header)) {
 		int hdr = self->header_func(self, buf);
 		if (!hdr) {
-			GstCaps *new_caps;
-			GstStructure *s;
-			int bps;
-
 			self->got_header = true;
-			if (gst_av_codec_open(av_ctx, self->codec) < 0) {
+			if (avcodec_open(self->av_ctx, self->codec) < 0) {
 				ret = GST_FLOW_ERROR;
 				goto leave;
 			}
 
-			bps = av_get_bits_per_sample_fmt(av_ctx->sample_fmt);
-			self->bps = bps;
+			{
+				GstCaps *new_caps;
 
-			s = gst_structure_new("audio/x-raw-int",
-					"rate", G_TYPE_INT, av_ctx->sample_rate,
-					"channels", G_TYPE_INT, av_ctx->channels,
-					"endianness", G_TYPE_INT, G_BYTE_ORDER,
-					"width", G_TYPE_INT, bps,
-					NULL);
-
-			switch (av_ctx->sample_fmt) {
-			case AV_SAMPLE_FMT_S16:
-			case AV_SAMPLE_FMT_S32:
-				gst_structure_set(s,
+				new_caps = gst_caps_new_simple("audio/x-raw-int",
+						"rate", G_TYPE_INT, self->av_ctx->sample_rate,
 						"signed", G_TYPE_BOOLEAN, TRUE,
-						"depth", G_TYPE_INT, bps,
+						"channels", G_TYPE_INT, self->av_ctx->channels,
+						"endianness", G_TYPE_INT, G_BYTE_ORDER,
+						"width", G_TYPE_INT, 16,
+						"depth", G_TYPE_INT, 16,
 						NULL);
-				break;
-			default:
-				break;
+
+				GST_INFO_OBJECT(self, "caps are: %" GST_PTR_FORMAT, new_caps);
+				gst_pad_set_caps(self->srcpad, new_caps);
+				gst_caps_unref(new_caps);
 			}
-
-			new_caps = gst_caps_new_full(s, NULL);
-
-			GST_INFO_OBJECT(self, "caps are: %" GST_PTR_FORMAT, new_caps);
-			gst_pad_set_caps(self->srcpad, new_caps);
-			gst_caps_unref(new_caps);
 		}
 	}
 
@@ -238,26 +209,19 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 		memcpy(pkt.data, buf->data, buf->size);
 		memset(pkt.data + pkt.size, 0, FF_INPUT_BUFFER_PADDING_SIZE);
 
-		if (G_UNLIKELY(self->timestamp == GST_CLOCK_TIME_NONE)) {
-			self->next_timestamp = self->timestamp = buf->timestamp;
-		} else if (self->next_timestamp != buf->timestamp) {
-			GST_WARNING_OBJECT(self, "reseting timestamp");
-			self->next_timestamp = self->timestamp = buf->timestamp;
-		}
-
-		self->next_timestamp += buf->duration;
+		if (G_UNLIKELY(self->timestamp == GST_CLOCK_TIME_NONE))
+			self->timestamp = buf->timestamp;
 
 		do {
 			void *buffer_data;
 			int buffer_size;
 			int read;
-			unsigned total_buffer_size;
 
 			buffer_data = self->buffer_data + self->ring.in;
 			buffer_size = self->buffer_size - self->ring.in;
-			read = avcodec_decode_audio3(av_ctx, buffer_data, &buffer_size, &pkt);
+			read = avcodec_decode_audio3(self->av_ctx, buffer_data, &buffer_size, &pkt);
 			if (read < 0) {
-				GST_WARNING_OBJECT(self, "error: %i", read);
+				ret = GST_FLOW_ERROR;
 				break;
 			}
 
@@ -270,14 +234,9 @@ pad_chain(GstPad *pad, GstBuffer *buf)
 				self->ring.out = 0;
 			}
 
-			if (BUFFER_SIZE > 0)
-				total_buffer_size = BUFFER_SIZE;
-			else
-				total_buffer_size = buffer_size;
-
-			if (self->ring.in - self->ring.out >= total_buffer_size) {
+			if (self->ring.in - self->ring.out >= BUFFER_SIZE) {
 				GstBuffer *out_buf;
-				out_buf = gst_buffer_new_and_alloc(total_buffer_size);
+				out_buf = gst_buffer_new_and_alloc(BUFFER_SIZE);
 				memcpy(out_buf->data, self->buffer_data + self->ring.out, out_buf->size);
 				calculate_timestamp(self, out_buf);
 				gst_buffer_set_caps(out_buf, self->srcpad->caps);
@@ -326,7 +285,7 @@ change_state(GstElement *element, GstStateChange transition)
 	switch (transition) {
 	case GST_STATE_CHANGE_READY_TO_NULL:
 		if (self->av_ctx) {
-			gst_av_codec_close(self->av_ctx);
+			avcodec_close(self->av_ctx);
 			av_freep(&self->av_ctx);
 		}
 		av_free_packet(&self->pkt);
@@ -386,11 +345,10 @@ sink_setcaps(GstPad *pad, GstCaps *caps)
 		GstBuffer *buf;
 
 		codec_id = CODEC_ID_FLAC;
-		self->header_func = flac_header;
 
 		stream_header = gst_structure_get_value(in_struc, "streamheader");
 		if (!stream_header)
-			return true;
+			return false;
 
 		stream_info = gst_value_array_get_value(stream_header, 0);
 
@@ -426,8 +384,8 @@ generate_src_template(void)
 			"rate", GST_TYPE_INT_RANGE, 8000, 96000,
 			"signed", G_TYPE_BOOLEAN, TRUE,
 			"endianness", G_TYPE_INT, G_BYTE_ORDER,
-			"width", GST_TYPE_INT_RANGE, 16, 32,
-			"depth", GST_TYPE_INT_RANGE, 16, 32,
+			"width", G_TYPE_INT, 16,
+			"depth", G_TYPE_INT, 16,
 			"channels", GST_TYPE_INT_RANGE, 1, 256,
 			NULL);
 
